(function () {
    var rally = require("rally");
    var request = require("request-promise").defaults({ jar: true });


    queryUtils = rally.util.query;
    refUtils = rally.util.ref;
    logger = require("./Logger.js");
    rallylog = require("./RallyLog.js");

    var apiKey;

    if (process.env.apiKey != undefined) {
        apiKey = process.env.apiKey;
        rallylog.info('apikey is ' + "\'" + apiKey + "\'");
    }
    else {
        rallylog.info('no apiKey defined');
        return;
    }

    // var project = process.env.project;
    // var ws = process.env.rallyworkspace;

    var proxy = process.env.rallyProxy;

    var tags = process.env.tag;

    if (process.env.method != undefined)
        var method = process.env.method;
    else
        var method = "Automated";   // default method type

    var baseUrl = "https://rally1.rallydev.com/slm/webservice/v2.0/";

    var type;

    if (process.env.type != undefined) {
        type = process.env.type;
        if (type != "Acceptance" && type != "Functional" && type != "Performance" && type != "Regression" && type != "Usability" && type != "User Interface") {
            console.log('Typo in the .env file type= : setting Type to default value "Acceptance"');
            rallylog.info('Typo in the .env file type= ' + "\'" + type + "\'" + ' setting Type to default value "Acceptance"');
            type = "Acceptance"
        }
    } else {
        type = "Acceptance"
    }

    restApi = rally({
        apiKey: apiKey,
        apiVersion: "v2.0",
        server: "https://rally1.rallydev.com",
        requestOptions: {
            headers: {
                "X-RallyIntegrationName": "Rally Result Logger for ProUI",
                "X-RallyIntegrationVendor": "GE Digital",
                "X-RallyIntegrationVersion": "1.0"
            }, proxy: proxy,jar: true
        }
    });

    var RallyLoggerWeb = function () {
        return {
            getObjectRef: function (objType, objName, project, ws, testfolder, callback) {
                if (objName == "") {
                    console.log("No workspace/project/testplan provided - using default");
                    logger.info("No workspace/project/testplan provided - using default");
                    callback("")
                } else {
                    var qr = queryUtils.where("name", "=", objName);
                    restApi.query({
                        type: objType,
                        start: 1,
                        pageSize: 200,
                        limit: Infinity,
                        fetch: ["FormattedID", "Name", "TestCases", "ObjectID", "LastResult", "RevisionHistory", "Steps", "Parent", "TestFolder", "Project"],
                        query: qr,
                        scope: {
                            workspace: ws,
                            project: project,
                            up: false,
                            down: false
                        }
                    }, function (error, result) {
                        if (error) {
                            console.log(error);
                            throw error
                        } else {
                            var last_run = false;
                            var test_case_result_id = 0;
                            var revision_history_ref = "";
                            var obj = JSON.parse(JSON.stringify(result.Results));
                            if (obj.length > 0 && objType == "testcase") {
                                var emptyTestFolder = new Array;
                                var result_index = -1;
                                var stepMatch = function (i, done) {
                                    if (i < obj.length) {
                                        var TestFolderobj = obj[i].TestFolder;
                                        if (TestFolderobj == null) {
                                            emptyTestFolder.push(i);
                                            stepMatch(i + 1)
                                        } else {
                                            var TestFolder = obj[i].TestFolder.ObjectID;
                                            if (typeof testfolder != "undefined" && TestFolder != null && TestFolder == testfolder) {
                                                result_index = i;
                                                stepMatch(obj.length)
                                            } else if (TestFolder == null) {
                                                emptyTestFolder.push(i);
                                                stepMatch(i + 1)
                                            } else if (i == obj.length - 1 && emptyTestFolder.length == 0) {
                                                callback("NOT_FOUND")
                                            } else {
                                                stepMatch(i + 1)
                                            }
                                        }
                                    } else {
                                        if (result_index == -1) {
                                            result_index = emptyTestFolder.pop()
                                        }
                                        if (obj[result_index].LastResult != undefined) {
                                            if (obj[result_index].LastResult == null) {
                                                last_run = false
                                            } else {
                                                last_run = true;
                                                test_case_result_id = obj[result_index].LastResult._ref;
                                                test_case_result_id = test_case_result_id.split("/");
                                                test_case_result_id = test_case_result_id[7]
                                            }
                                        } else {
                                            last_run = false
                                        }
                                        if (obj[result_index].RevisionHistory != undefined) {
                                            revision_history_ref = obj[result_index].RevisionHistory._ref
                                        }
                                        var myref = obj[result_index]._ref;
                                        var id = result.Results[result_index].ObjectID;
                                        var ref = refUtils.getRelative(myref);
                                        callback({
                                            ref: ref,
                                            id: id,
                                            last_run: last_run,
                                            test_case_result_id: test_case_result_id,
                                            revision_history_ref: revision_history_ref
                                        })
                                    }
                                };
                                stepMatch(0)
                            } else if (obj.length > 0 && objType == "testfolder") {
                                var emptyTestFolder = new Array;
                                var result_index = -1;
                                var stepMatch = function (i, done) {
                                    if (i < obj.length) {
                                        var TestFolderobj = obj[i].Parent;
                                        if (TestFolderobj == null) {
                                            emptyTestFolder.push(i);
                                            stepMatch(i + 1)
                                        } else {
                                            var TestFolder = obj[i].Parent.ObjectID;
                                            if (typeof testfolder != "undefined" && TestFolder != null && TestFolder == testfolder) {
                                                result_index = i;
                                                stepMatch(obj.length)
                                            } else if (TestFolder == null) {
                                                emptyTestFolder.push(i);
                                                stepMatch(i + 1)
                                            } else if (i == obj.length - 1 && emptyTestFolder.length == 0) {
                                                callback("NOT_FOUND")
                                            } else {
                                                stepMatch(i + 1)
                                            }
                                        }
                                    } else {
                                        var parent;
                                        if (result_index == -1) {
                                            result_index = emptyTestFolder.pop();
                                            parent = obj[result_index].Parent
                                        } else {
                                            parent = obj[result_index].Parent.ObjectID
                                        }
                                        if (obj[result_index].LastResult != undefined) {
                                            if (obj[result_index].LastResult == null) {
                                                last_run = false
                                            } else {
                                                last_run = true;
                                                test_case_result_id = obj[result_index].LastResult._ref;
                                                test_case_result_id = test_case_result_id.split("/");
                                                test_case_result_id = test_case_result_id[7]
                                            }
                                        } else {
                                            last_run = false
                                        }
                                        if (obj[result_index].RevisionHistory != undefined) {
                                            revision_history_ref = obj[result_index].RevisionHistory._ref
                                        }
                                        var myref = obj[result_index]._ref;
                                        var id = result.Results[result_index].ObjectID;
                                        var ref = refUtils.getRelative(myref);
                                        callback({
                                            ref: ref,
                                            id: id,
                                            last_run: last_run,
                                            parent: parent,
                                            test_case_result_id: test_case_result_id,
                                            revision_history_ref: revision_history_ref
                                        })
                                    }
                                };
                                stepMatch(0)
                            } else {
                                callback("NOT_FOUND")
                            }
                        }
                    })
                }
            },

            UserPermissions: function (type, object, callback) {
                var proj_ref = '';
                var ws_ref = '';
                restApi.query({
                    type: 'userpermission', //the type to query
                    start: 1, //the 1-based start index, defaults to 1
                    pageSize: 200, //the page size (1-200, defaults to 200)
                    limit: Infinity, //the maximum number of results to return- enables auto paging
                    fetch: ['Workspace', 'Name', 'Project', 'ObjectID', 'User'], //the fields to retrieve
                    scope: {}
                }, function (error, result) {
                    if (error) {
                        console.error(error);
                        callback(error);
                    } else {
                        var obj = JSON.parse(JSON.stringify(result.Results));
                        if (obj.length > 0) {
                            var myref = obj[0].User._ref;
                            var ref = refUtils.getRelative(myref);
                            var owner_id = obj[0].User.ObjectID;
                            for (var i = 0; i < obj.length; i++) {
                                if (type == 'workspace') {
                                    if (obj[i].Workspace.Name == object) {
                                        //console.log('found workspace - user has permissions');
                                        ws_ref = obj[i].Workspace._ref;
                                        ws_ref = refUtils.getRelative(ws_ref);
                                        break;
                                    }
                                } else if (type == 'project') {
                                    if (obj[i].Project != undefined) {
                                        if (obj[i].Project.Name == object.project && obj[i].Workspace.Name == object.workspace) {
                                            //console.log('found project - user has permissions');
                                            proj_ref = obj[i].Project._ref;
                                            proj_ref = refUtils.getRelative(proj_ref);
                                            break;
                                        }
                                    }
                                }
                            }
                            if ((i == obj.length) && (object != '')) {
                                callback('NOT_FOUND');
                            } else {
                                callback({owner_id: owner_id, wsRef: ws_ref, projRef: proj_ref});
                            }
                        } else {
                            callback('NOT_FOUND');
                        }
                    }
                });
            },

            getUSTSRef: function (USTS, ws, proj, type, callback) {
                if (USTS == "") {
                    callback({id: ""})
                } else {
                    var qr = queryUtils.where("FormattedID", "=", USTS);
                    restApi.query({
                        type: type,
                        start: 1,
                        pageSize: 200,
                        limit: Infinity,
                        fetch: ["Name", "ObjectID"],
                        query: qr,
                        scope: {workspace: ws, project: proj}
                    }, function (error, result) {
                        if (error) {
                            console.log(error);
                            throw error
                        } else {
                            var obj = JSON.parse(JSON.stringify(result.Results));
                            // console.log("in this finding test set obj is: "+"type: "+type+" "+ JSON.stringify(obj))
                            if (obj.length > 0) {
                                var myref = obj[0]._ref;
                                var ref = refUtils.getRelative(myref);
                                var id = obj[0].ObjectID;
                                callback({ref: ref, id: id})
                            } else {
                                ref = "";
                                id = "";
                                callback({ref: ref, id: id})
                            }
                        }
                    })
                }
            },

            createTestPlan: function (name, ws, proj, rootid, callback) {
                var body, folder_id;
                if (typeof rootid == "undefined" || rootid.length == 0) {
                    body = "{\n" + '    "testfolder":\n' + "         {\n" + '              "Name":"' + name + '",\n' + '              "Workspace":"' + ws + '",\n' + '              "Project":"' + proj + '"\n' + "         }\n" + "}"
                } else {
                    body = "{\n" + '    "testfolder":\n' + "         {\n" + '              "Name":"' + name + '",\n' + '              "Parent":"' + "/testfolder/" + rootid + '",\n' + '              "Workspace":"' + ws + '",\n' + '              "Project":"' + proj + '"\n' + "         }\n" + "}"
                }
                var options = {
                    method: "POST",
                    url: baseUrl + "testfolder/create?fetch=true",
                    headers: {zsessionid: apiKey},
                    body: body,
                    proxy: proxy
                };
                request(options).then(function (body, error) {
                    if (error) {
                        console.log("Rally error: " + error);
                        rallylog.info("Error in creating Test Plan: " + error);
                        return callback()
                    } else {
                        var json_body = JSON.parse(body);
                        if(json_body.CreateResult.Errors.length>0) {
                            console.log("Error in creating Test Plan: Could not parse Feature Name. If you have a Rootfolder, make sure you are not missing the colon after rootfolder is stated. Ex. RF-test:feature name");
                            rallylog.info("Error in creating Test Plan: " + "Rally error: Could not parse Feature Name. If you have a Rootfolder, make sure you are not missing the colon after rootfolder is stated. Ex. RF-test:feature name");
                            return callback()
                        }
                        folder_id = json_body.CreateResult.Object.ObjectID;
                        rallylog.info("Created Test Plan (" + json_body.CreateResult.Object.FormattedID + "): " + name + "\n");
                        return callback({id: folder_id})
                    }
                })
            },

            updateTestPlan: function (name, ws, proj, rootid, curfolderid, callback) {
                var body = null;
                if (typeof rootid == "undefined" || rootid == "") {
                    body = "{\n" + '    "testfolder":\n' + "         {\n" + '              "Name":"' + name + '",\n' + '              "Workspace":"' + ws + '",\n' + '              "Project":"' + proj + '"\n' + "         }\n" + "}"
                } else {
                    body = "{\n" + '    "testfolder":\n' + "         {\n" + '              "Name":"' + name + '",\n' + '              "Parent":"' + "/testfolder/" + rootid + '",\n' + '              "Workspace":"' + ws + '",\n' + '              "Project":"' + proj + '"\n' + "         }\n" + "}"
                }
                var options = {
                    method: "POST",
                    url: baseUrl + "testfolder/" + curfolderid,
                    headers: {zsessionid: apiKey},
                    proxy: proxy,
                    body: body
                };
                request(options).then(function (body, error) {
                    if (error) {
                        console.log("Rally error: " + error);
                        rallylog.info("Error in updating Test Plan(" + json_body.CreateResult.Object.FormattedID + "): " + error)
                        throw error
                    } else {
                        var json_body = JSON.parse(body);
                        if (json_body.OperationResult.Errors.length > 0) {
                            var error = "Rally error: " + JSON.stringify(json_body.OperationResult.Errors);
                            rallylog.info("Error in updating Test Plan(" + json_body.CreateResult.Object.FormattedID + "): " + error);
                            throw error
                        }
                        rallylog.info("Updated Test Plan (" + json_body.OperationResult.Object.FormattedID + "): " + name + "\n");
                        folder_id = json_body.OperationResult.Object.ObjectID;
                        return callback({id: folder_id})
                    }
                })
            },

            getTagId: function (ws, project, tags, callback) {
                var qr = queryUtils.where("name", "=", tags);
                restApi.query({
                    type: "Tag",
                    start: 1,
                    pageSize: 200,
                    limit: Infinity,
                    fetch: ['FormattedID'],
                    query: qr,
                    scope: {
                        workspace: ws,
                        project: project,
                        up: true,
                        down: true
                    }
                }, function (error, result) {
                    if (error) {
                        console.log("Error in tags: " + error);
                        rallylog.info("Error in retrieving Tag ID: " + error)
                        return callback(error)
                    } else {
                        //console.log("Here are all the tags: "+ JSON.stringify(result));
                        //console.log("result.Results.length: "+ result.Results.length);
                        //console.log("Error: "+error)
                        if (result.Results.length <= 0) {
                            console.log("Could not attach tag because there are no tags associated with '" + tags + "'")
                            rallylog.info("Could not attach tag because there are no tags associated with '" + tags + "'\n")
                            callback("There are no tags associated with " + tags);
                        }
                        else {

                            tag_id = result.Results[0]._ref;

                            tag_id = tag_id.match(/(\d)*$/g)

                            tag_id = tag_id + "";

                            tag_id = tag_id.replace(/,/g, "");
                            //console.log("The tag id: "+tag_id)
                            return callback({tagID: tag_id})
                        }
                    }

                });
            },

            setTagId: function (obj, ws, project, callback) {
                //console.log("INsdie set tag")
                //console.log("obj.glb_test_case_id: "+obj.glb_test_case_id)
                //console.log("obj.glb_tag "+obj.tagId)
                restApi.add({
                    ref: '/testcase/' + obj.glb_test_case_id, //the object which owns the collection- in this case the parent story
                    collection: 'Tags', //the name of the collection being added to
                    data: [
                        {_ref: '/tag/' + obj.tagId} //add this existing defect
                    ],
                    scope: {
                        workspace: ws,
                        project: project,
                        up: true,
                        down: true
                    },
                    fetch: [], //the fields to retrieve in the results
                    requestOptions: {} //optional additional options to pass through to request
                }, function (error, result) {
                    if (error) {
                        console.log(error);
                        rallylog.info("Error in setting tag id: " + error + "\n");
                        return callback(error)
                    } else {
                        rallylog.info("Set tag: " + result.Results[0].Name + "\n");
                        return callback()
                    }

                });

            },

            createTestcase: function (opts, ws, project, callback) {
                if (opts.kObj.duration == undefined) {
                    opts.kObj.duration = 0
                }
                if (opts.tcObj.glb_folder_id == null) {
                    opts.tcObj.glb_folder_id = 0
                }
                var create_id = 0;
                var to_ret, body;
                if (opts.tcObj.test_case_description == "") {
                    opts.tcObj.stepsArray = opts.tcObj.stepsArray.toString().split('"').join("");
                    opts.tcObj.test_case_description = "<br>" + opts.tcObj.stepsArray
                } else {
                    opts.tcObj.stepsArray = opts.tcObj.stepsArray.toString().split('"').join("");
                    opts.tcObj.test_case_description = opts.tcObj.test_case_description + "<br>" + opts.tcObj.stepsArray
                }
                body = "{\n" + '    "testcase":\n' + "        {\n" + '            "Name":"' + opts.tcObj.test_case_name.replace(/\s+\s/i, "") + '",\n' + '            "Description":"' + opts.tcObj.test_case_description + '",\n' + '    "Method":"' + method + '",\n' + '            "TestFolder":"' + opts.tcObj.glb_folder_id + '",\n' + '            "WorkProduct":"' + opts.tcObj.User_Story + '",\n' + '            "Owner":"' + opts.tcObj.owner + '",\n' + '            "Notes":"' + opts.error + '",\n' + '            "Type":"' + type + '",\n' + '            "Workspace":"' + ws + '",\n' + '            "Project":"' + project + '"\n' + "        }\n" + "}";
                var options = {
                    method: "POST",
                    url: baseUrl + "testcase/create",
                    headers: {zsessionid: apiKey},
                    proxy: proxy,
                    body: body
                };
                request(options).then(function (body, error) {
                    if (error) {
                        console.log("Rally error: " + error);
                        rallylog.info("Error in creating Test Case: " + error + "\n");
                        return callback({id: ""})
                    } else {
                        body = JSON.parse(body);
                        if (body.CreateResult.Object != undefined) {
                            opts.tcObj.glb_test_case_id = body.CreateResult.Object.ObjectID
                            rallylog.info("Created Test Case (" + body.CreateResult.Object.FormattedID + "): " + opts.tcObj.test_case_name + "\n");
                        } else {
                            opts.tcObj.glb_test_case_id = 0;
			    if (body.CreateResult.Errors && body.CreateResult.Errors.length > 0) {
                                console.log(body.CreateResult.Errors.toString());
                                rallylog.info(body.CreateResult.Errors.toString());
                            }
                        }
                        return callback({id: opts.tcObj.glb_test_case_id})
                    }
                })
            },

            createTestcaseResult: function (opts, ws, proj, callback) {

                if (opts.resp.duration == undefined || opts.resp.duration == "undefined" || opts.resp.duration == null || opts.resp.duration == NaN || opts.resp.duration == "NaN") {
                    opts.resp.duration = 0
                } else if (opts.resp.duration > 9999999999) {
                    opts.resp.duration = 9999999999
                }
                if (opts.resp.status == "passed") {
                    opts.resp.status = "Pass"
                } else if (opts.resp.status == "failed") {
                    opts.resp.status = "Fail"
                } else if (opts.resp.status == "skipped") {
                    opts.resp.status = "Inconclusive";
                    duration = 0
                }
                var options = {
                    method: "POST",
                    url: baseUrl + "testcaseresult/create",
                    headers: {zsessionid: apiKey},
                    proxy: proxy,
                    body: '{\n    "testcaseresult":\n' + "        {\n" + '            "TestCase":"/testcase/' + opts.tcObj.glb_test_case_id + '",\n' + '            "Build":"' + opts.tcObj.build_num + '",\n' + '            "Duration":"' + opts.resp.duration + '",\n' + '            "Verdict":"' + opts.resp.status + '",\n' + '            "Date":"' + (new Date).toISOString() + '",\n' + '            "TestSet":"' + opts.resp.id + '",\n' + '            "Workspace":"' + ws + '"\n' + "        }\n" + "}"
                };

                // console.log("in create test result the posting options are: "+ JSON.stringify(options))
                var id = 0;

                request(options).then(function (body, error) {
                    var to_return;
                    if (error) {
                        console.log("Rally error: " + error)
                        rallylog.info("Error in creating test case result for " + opts.tcObj.glb_test_case_id + "\n");
                    } else {
                        body = JSON.parse(body);
                        var err = body.CreateResult.Errors;
                        if (err != "") {
                            id = null;
                            return callback({id: ""})
                        } else {
                            rallylog.info("Created Test Case Result for " + body.CreateResult.Object.TestCase._refObjectName + ": " + opts.resp.status + "\n\n");
                            id = body.CreateResult.Object.ObjectID;
                            return callback({id: id})
                        }
                    }
                })
            },

            updateTestCaseResult: function (tcObj, resp, callbck) {
                if (resp.duration == undefined) {
                    resp.duration = 0
                }
                if (resp.duration != undefined && resp.duration > 9999999999) {
                    resp.duration = 9999999999
                }
                if (resp.status == "passed") {
                    resp.status = "Pass"
                } else if (resp.status == "failed") {
                    resp.status = "Fail"
                } else if (resp.status == "skipped") {
                    resp.status = "Inconclusive";
                    resp.duration = 0
                }
                var to_return;
                var options = {
                    method: "POST",
                    url: baseUrl + "testcaseresult/" + tcObj.glb_test_result_id,
                    headers: {zsessionid: apiKey},
                    proxy: proxy,
                    body: "{\n" + '    "testcaseresult":\n' + "        {\n" + '            "Build":"' + tcObj.build_num + '",\n' + '            "Duration":"' + resp.duration + '",\n' + '            "Date":"' + (new Date).toISOString() + '",\n' + '            "TestCase":"/testcase/' + tcObj.glb_test_case_id + '",\n' + '            "Verdict":"' + resp.status + '",\n' + '            "TestSet":"' + resp.id + '"\n' + "        }\n" + "}"
                };
                request(options).then(function (body, error) {
                    if (error) {
                        console.log("Rally error: " + error)
                    } else {
                        to_return = {i: resp.i, j: resp.j};
                        return callbck(to_return)
                    }
                })
            },

            updateTestCase: function (opts, ws, proj, callback) {
                if (opts.tcObj.test_case_description == "") {
                    opts.tcObj.stepsArray = opts.tcObj.stepsArray.toString().split('"').join("");
                    opts.tcObj.test_case_description = "N/A<br>" + opts.tcObj.stepsArray
                } else {
                    opts.tcObj.stepsArray = opts.tcObj.stepsArray.toString().split('"').join("");
                    opts.tcObj.test_case_description = opts.tcObj.test_case_description + "<br>" + opts.tcObj.stepsArray
                }
                var request = require("request-promise").defaults({ jar: true });
                var options = {
                    method: "POST",
                    url: baseUrl + "testcase/" + opts.tcObj.glb_test_case_id,
                    headers: {zsessionid: apiKey},
                    proxy: proxy,
                    body: "{\n" + '    "testcase":\n        {\n' + '            "WorkProduct":"' + opts.resp.id + '",\n' + '            "TestFolder":"' + opts.tcObj.glb_folder_id + '",\n' + '            "Owner":"' + opts.owner + '",\n' + '    "Method":"' + method + '",\n' + '            "Notes":"' + opts.error + '",\n' + '            "Type":"' + type + '",\n' + '            "Description":"' + opts.tcObj.test_case_description + '"\n' + "        }\n" + "}"
                };
                request(options).then(function (body, error) {
                    if (error) {
                        console.log("error: " + error);
                        rallylog.info("Error in updating test case: " + error + "\n");
                    }
                    else {
                        body = JSON.parse(body);
                        if (body.OperationResult.Object != undefined) {
                        rallylog.info("Updated Test Case (" + body.OperationResult.Object.FormattedID + "): " + body.OperationResult.Object._refObjectName + "\n");
                        }
                        else
                        {
                            if (body.OperationResult.Errors && body.OperationResult.Errors.length > 0) {
                                console.log(body.OperationResult.Errors.toString());
                                rallylog.info(body.OperationResult.Errors.toString());
                            }
                        }
                    }
                    callback();
                })
            },

            updateTestSet: function (tcObj, resp, callbck) {
                var to_return;
                if (resp.id == "") {
                    to_return = {
                        id: "",
                        status: resp.status,
                        duration: resp.duration,
                        glb_test_case_id: resp.glb_test_case_id,
                        glb_folder_id: tcObj.glb_rootfolder_id,
                        json: resp.json,
                        i: resp.i,
                        j: resp.j,
                        k: k
                    };
                    callbck(to_return)
                } else {
                    to_return = {
                        id: resp.id,
                        status: resp.status,
                        duration: resp.duration,
                        glb_test_case_id: resp.glb_test_case_id,
                        glb_folder_id: tcObj.glb_folder_id,
                        json: resp.json,
                        i: resp.i,
                        j: resp.j,
                        k: k
                    };
                    var options = {
                        method: "POST",
                        url: "https://rally1.rallydev.com/slm/webservice/v2.x/TestSet/" + resp.id + "/TestCases/add?fetch=true",
                        headers: {zsessionid: apiKey},
                        proxy: proxy,
                        body: '{"CollectionItems":[{"_ref":"/testcase/' + tcObj.glb_test_case_id + '"}]}'
                    };
                    request(options).then(function (body, error) {
                        body = JSON.parse(body);
                        // rallylog.info("Updated Test Set ("+body.OperationResult.Results[0].FormattedID+"): "+body.OperationResult.Results[0].Name+"\n");
                        // rallylog.info("Added Test Set to: "+body.CreateResult.Object.FormattedID+"\n");
                        // console.log("The updated test set body: "+JSON.stringify(body))
                        // console.log("error: "+JSON.stringify(error))
                        callbck(to_return)
                    })
                }
            }
        }
    };
    module.exports = new RallyLoggerWeb
})();